@@ -107,7 +107,7 @@ class BaseParser:
             if self.unknown_lines:
                 print(f"Could not parse {len(self.unknown_lines)} lines")
             else:
-                print(f"All lines where parsed")
+                print("All lines where parsed")
             print(f"Found {len(self.flags)} flags")
             print(f"Found {len(self.enums)} enums")
             print(f"Found {len(self.structs)} structs")
@@ -315,8 +315,10 @@ class IdlParser(BaseParser):
                         t = "int"
                     elif arg_type in ["boolean"]:
                         t = "bool"
-                    elif arg_type in ["DOMString", "DOMString?"]:
+                    elif arg_type in ["DOMString", "DOMString?", "USVString"]:
                         t = "str"
+                    elif arg_type in ["object"]:
+                        t = "dict"
                     elif arg_type.startswith("GPU"):
                         t = arg_type
                     elif arg_type.startswith("sequence<GPU"):
