@@ -20,7 +20,7 @@ logger = logging.getLogger("wgpu")
 
 
 # wgpu.help('RequestAdapterOptions', 'requestadapter', dev=True)
-# IDL: Promise<GPUAdapter> requestAdapter(optional GPURequestAdapterOptions options = {});
+# IDL: Promise<GPUAdapter?> requestAdapter(optional GPURequestAdapterOptions options = {});
 def request_adapter(*, canvas, power_preference: "GPUPowerPreference"):
     """ Get a :class:`GPUAdapter`, the object that represents an abstract wgpu
     implementation, from which one can request a :class:`GPUDevice`.
@@ -38,7 +38,7 @@ def request_adapter(*, canvas, power_preference: "GPUPowerPreference"):
 
 
 # wgpu.help('RequestAdapterOptions', 'requestadapter', dev=True)
-# IDL: Promise<GPUAdapter> requestAdapter(optional GPURequestAdapterOptions options = {});
+# IDL: Promise<GPUAdapter?> requestAdapter(optional GPURequestAdapterOptions options = {});
 async def request_adapter_async(*, canvas, power_preference: "GPUPowerPreference"):
     """ Async version of ``request_adapter()``.
     """
@@ -72,7 +72,7 @@ class GPUAdapter:  # Not a GPUObject
         return self._extensions
 
     # wgpu.help('DeviceDescriptor', 'adapterrequestdevice', dev=True)
-    # IDL: Promise<GPUDevice> requestDevice(optional GPUDeviceDescriptor descriptor = {});
+    # IDL: Promise<GPUDevice?> requestDevice(optional GPUDeviceDescriptor descriptor = {});
     def request_device(
         self,
         *,
@@ -90,7 +90,7 @@ class GPUAdapter:  # Not a GPUObject
         raise NotImplementedError()
 
     # wgpu.help('DeviceDescriptor', 'adapterrequestdevice', dev=True)
-    # IDL: Promise<GPUDevice> requestDevice(optional GPUDeviceDescriptor descriptor = {});
+    # IDL: Promise<GPUDevice?> requestDevice(optional GPUDeviceDescriptor descriptor = {});
     async def request_device_async(
         self,
         *,
@@ -149,6 +149,8 @@ class GPUObject:
     def __del__(self):
         self._destroy()
 
+    # Public destroy() methods are implemented on classes as the WebGPU spec specifies.
+
 
 class GPUDevice(GPUObject):
     """
@@ -169,6 +171,7 @@ class GPUDevice(GPUObject):
         self._extensions = tuple(sorted([str(x) for x in extensions]))
         self._limits = limits.copy()
         self._default_queue = default_queue
+        default_queue._device = self  # because it could not be set earlier
 
     @property
     def extensions(self):
@@ -191,40 +194,36 @@ class GPUDevice(GPUObject):
 
     # wgpu.help('BufferDescriptor', 'devicecreatebuffer', dev=True)
     # IDL: GPUBuffer createBuffer(GPUBufferDescriptor descriptor);
-    def create_buffer(self, *, label="", size: int, usage: "GPUBufferUsageFlags"):
+    def create_buffer(
+        self,
+        *,
+        label="",
+        size: int,
+        usage: "GPUBufferUsageFlags",
+        mapped_at_creation: bool = False,
+    ):
         """ Create a :class:`GPUBuffer` object.
 
         Arguments:
             label (str): A human readable label. Optional.
             size (int): The size of the buffer in bytes.
             usage (BufferUsageFlags): The ways in which this buffer will be used.
+            mapped_at_creation (bool): Must be False, because we currently
+                use read_data() and write_data() instead of buffer mapping.
         """
         raise NotImplementedError()
 
-    # wgpu.help('BufferDescriptor', 'devicecreatebuffermapped', dev=True)
-    # IDL: GPUMappedBuffer createBufferMapped(GPUBufferDescriptor descriptor);
-    def create_buffer_mapped(
-        self, *, label="", size: int, usage: "GPUBufferUsageFlags"
-    ):
-        """ Create a :class:`GPUBuffer` object that is mapped from the start. It must
-        be unmapped before using it in a pipeline.
+    def create_buffer_with_data(self, *, label="", data, usage: "GPUBufferUsageFlags"):
+        """ Create a :class:`GPUBuffer` object initialized with the given data.
 
         Arguments:
             label (str): A human readable label. Optional.
-            size (int): The size of the buffer in bytes.
+            data: Any object supporting the Python buffer protocol (this
+                includes bytes, bytearray, ctypes arrays, numpy arrays, etc.).
             usage (BufferUsageFlags): The ways in which this buffer will be used.
         """
         raise NotImplementedError()
 
-    # wgpu.help('BufferDescriptor', 'devicecreatebuffermapped', dev=True)
-    # IDL: GPUMappedBuffer createBufferMapped(GPUBufferDescriptor descriptor);
-    async def create_buffer_mapped_async(
-        self, *, label="", size: int, usage: "GPUBufferUsageFlags"
-    ):
-        """ Async version of ``create_buffer_mapped()``.
-        """
-        raise NotImplementedError()
-
     # wgpu.help('TextureDescriptor', 'devicecreatetexture', dev=True)
     # IDL: GPUTexture createTexture(GPUTextureDescriptor descriptor);
     def create_texture(
@@ -406,11 +405,11 @@ class GPUDevice(GPUObject):
 
     # wgpu.help('ShaderModuleDescriptor', 'devicecreateshadermodule', dev=True)
     # IDL: GPUShaderModule createShaderModule(GPUShaderModuleDescriptor descriptor);
-    def create_shader_module(self, *, label="", code: str):
+    def create_shader_module(self, *, label="", code: str, source_map: "dict" = None):
         """ Create a :class:`GPUShaderModule` object from shader source.
 
         Currently, only SpirV is supported. One can compile glsl shaders to
-        SpirV ahead of time, or use the python-shader package to write shaders
+        SpirV ahead of time, or use the pyshader package to write shaders
         in Python.
 
         Arguments:
@@ -426,7 +425,7 @@ class GPUDevice(GPUObject):
         self,
         *,
         label="",
-        layout: "GPUPipelineLayout",
+        layout: "GPUPipelineLayout" = None,
         compute_stage: "GPUProgrammableStageDescriptor",
     ):
         """ Create a :class:`GPUComputePipeline` object.
@@ -444,7 +443,7 @@ class GPUDevice(GPUObject):
         self,
         *,
         label="",
-        layout: "GPUPipelineLayout",
+        layout: "GPUPipelineLayout" = None,
         vertex_stage: "GPUProgrammableStageDescriptor",
         fragment_stage: "GPUProgrammableStageDescriptor" = None,
         primitive_topology: "GPUPrimitiveTopology",
@@ -588,7 +587,7 @@ class GPUDevice(GPUObject):
         In the WebGPU spec this is a method of the canvas. In wgpu-py
         it's a method of the device.
 
-        Parameters:
+        Arguments:
             canvas (WgpuCanvasInterface): An object implementing the canvas interface.
             format (TextureFormat): The texture format, e.g. "bgra8unorm-srgb".
             usage (TextureUsage): Default ``TextureUsage.OUTPUT_ATTACHMENT``.
@@ -617,17 +616,17 @@ class GPUBuffer(GPUObject):
     :func:`GPUDevice.create_buffer_mapped` or :func:`GPUDevice.create_buffer_mapped_async`.
 
     One can sync data in a buffer by mapping it (or by creating a mapped
-    buffer) and then setting/getting the values in the mapped array.
+    buffer) and then setting/getting the values in the mapped memoryview.
     Alternatively, one can tell the GPU (via the command encoder) to
     copy data between buffers and textures.
     """
 
-    def __init__(self, label, internal, device, size, usage, state, mapping):
+    def __init__(self, label, internal, device, size, usage, state):
         super().__init__(label, internal, device)
         self._size = size
         self._usage = usage
         self._state = state
-        self._mapping = mapping
+        self._map_mode = 3 if state == "mapped at creation" else 0
 
     @property
     def size(self):
@@ -643,66 +642,81 @@ class GPUBuffer(GPUObject):
         """
         return self._usage
 
-    @property
-    def state(self):
-        """ The current state of the GPUBuffer:
+    # WebGPU specifies an API to sync data with the buffer via mapping.
+    # The idea is to (async) request mapped data, read from / write to
+    # this memory (using getMappedRange), and then unmap.  A buffer
+    # must be unmapped before it can be used in a pipeline.
+    #
+    # This means that the mapped memory is reclaimed (i.e. invalid)
+    # when unmap is called, and that whatever object we expose the
+    # memory with to the user, must be set to a state where it can no
+    # longer be used. I currently can't think of a good way to do this.
+    #
+    # So instead, we can use mapping internally to allow reading and
+    # writing but not expose it via the public API. The only
+    # disadvantage (AFAIK) is that there could be use-cases where a
+    # memory copy could be avoided when using mapping.
+    #
+    #
+    # IDL specifies getMappedRange, but there is no equivalent in wgpu yet
+    #
+    # @property
+    # def state(self):
+    #     """ The current state of the GPUBuffer:
+    #
+    #     * "mapped" when the buffer is available for CPU operations.
+    #     * "mapped at creation" where the GPUBuffer was just created and
+    #       is available for CPU operations on its content.
+    #     * "mapping pending" where the GPUBuffer is being made available
+    #       for CPU operations on its content.
+    #     * "unmapped" when the buffer is available for GPU operations.
+    #     * "destroyed", when the buffer is no longer available for any
+    #       operations except destroy.
+    #     """
+    #     return self._state
+    #
+    # @property
+    # def map_mode(self):
+    #     """ The map mode flag. Zero if unmapped.
+    #     """
+    #     return self._map_mode
+    #
+    # def map(self, mode, offset=0, size=0):
+    #     """ Make the buffer memory accessable to the CPU for reading or writing.
+    #     If size is zero, the remaining size (after offset) is used.
+    #     Returns the mapped memory as a memoryview, which can be mapped
+    #     to e.g. a ctypes array or numpy array.
+    #     """
+    #     raise NotImplementedError()
+    #
+    # async def map_async(self, mode, offset=0, size=0):
+    #     """ Async version of ``map()``.
+    #     """
+    #     raise NotImplementedError()
+    #
+    # def unmap(self):
+    #     """ Unmap the buffer so that it can be used in a GPU pipeline.
+    #     """
+    #     raise NotImplementedError()
 
-        * "mapped" when the buffer is available for CPU operations.
-        * "unmapped" when the buffer is available for GPU operations.
-        * "destroyed", when the buffer is no longer available for any
-          operations except destroy.
-        """
-        return self._state
-
-    # NOTE: this attribute is not specified by IDL, I think its still undecided how to
-    #       expose the memory
-    @property
-    def mapping(self):
-        """ The mapped memory of the buffer, exposed as a ctypes array.
-        Is only not ``None`` when the buffer is mapped. Can be cast to a
-        ctypes array of appropriate type using
-        ``your_array_type.from_buffer(b.mapping)``. Or use something
-        like ``np.frombuffer(b.mapping, np.float32)`` to map it to a
-        numpy array of appropriate dtype and shape.
-        """
-        return self._mapping
-
-    # wgpu.help('buffermapreadasync', dev=True)
-    # IDL: Promise<ArrayBuffer> mapReadAsync();
-    def map_read(self):
-        """ Make the buffer memory accessable to the CPU for reading.
-        Sets the ``mapping`` property and returns the mapped memory as
-        a ctypes array.
+    def read_data(self, offset=0, size=0):
+        """ Read buffer data. Returns the mapped memory as a memoryview,
+        which can be mapped to e.g. a ctypes array or numpy array.
+        If size is zero, the remaining size (after offset) is used.
+        The buffer usage must include MAP_READ.
         """
         raise NotImplementedError()
 
-    # wgpu.help('buffermapreadasync', dev=True)
-    # IDL: Promise<ArrayBuffer> mapReadAsync();
-    async def map_read_async(self):
-        """ Async version of ``map_read()``.
+    async def read_data_async(self, offset=0, size=0):
+        """ Asnc version of read_data().
         """
-        raise NotImplementedError()
 
-    # wgpu.help('buffermapwriteasync', dev=True)
-    # IDL: Promise<ArrayBuffer> mapWriteAsync();
-    def map_write(self):
-        """ Make the buffer memory accessable to the CPU for writing.
-        Sets the ``mapping`` property and returns the mapped memory as
-        a ctypes array.
-        """
-        raise NotImplementedError()
+    def write_data(self, data, offset=0):
+        """ Write data to the buffer. The data can be any object
+        supporting the buffer protocol. The buffer usage must include MAP_WRITE.
 
-    # wgpu.help('buffermapwriteasync', dev=True)
-    # IDL: Promise<ArrayBuffer> mapWriteAsync();
-    async def map_write_async(self):
-        """ Async version of ``map_write()``.
-        """
-        raise NotImplementedError()
-
-    # wgpu.help('bufferunmap', dev=True)
-    # IDL: void unmap();
-    def unmap(self):
-        """ Unmap the buffer so that it can be used in a GPU pipeline.
+        Note: this method deviates from the WebGPU spec, and it could be deprecated.
+        Better use ``device.create_buffer_with_data()`` or ``queue.write_buffer()``.
         """
         raise NotImplementedError()
 
@@ -726,6 +740,46 @@ class GPUTexture(GPUObject):
     Create a texture using :func:`GPUDevice.create_texture`.
     """
 
+    def __init__(self, label, internal, device, tex_info):
+        super().__init__(label, internal, device)
+        self._tex_info = tex_info
+
+    @property
+    def texture_size(self):
+        """ The size of the texture in mipmap level 0, as a 3-tuple of ints.
+        """
+        return self._tex_info["size"]
+
+    @property
+    def mip_level_count(self):
+        """ The total number of the mipmap levels of the texture.
+        """
+        return self._tex_info["mip_level_count"]
+
+    @property
+    def sample_count(self):
+        """ The number of samples in each texel of the texture.
+        """
+        return self._tex_info["sample_count"]
+
+    @property
+    def dimension(self):
+        """ The dimension of the texture.
+        """
+        return self._tex_info["dimension"]
+
+    @property
+    def format(self):
+        """ The format of the texture.
+        """
+        return self._tex_info["format"]
+
+    @property
+    def texture_usage(self):  # Not sure why there's a "texture_" prefix
+        """ The allowed usages for this texture.
+        """
+        return self._tex_info["usage"]
+
     # wgpu.help('TextureViewDescriptor', 'texturecreateview', dev=True)
     # IDL: GPUTextureView createView(optional GPUTextureViewDescriptor descriptor = {});
     def create_view(
@@ -775,6 +829,23 @@ class GPUTextureView(GPUObject):
     Create a texture view using :func:`GPUTexture.create_view`.
     """
 
+    def __init__(self, label, internal, device, texture, size):
+        super().__init__(label, internal, device)
+        self._texture = texture
+        self._size = size
+
+    @property
+    def size(self):
+        """ The texture size (as a 3-tuple).
+        """
+        return self._size
+
+    @property
+    def texture(self):
+        """ The texture object to which this is a view.
+        """
+        return self._texture
+
 
 class GPUSampler(GPUObject):
     """
@@ -833,8 +904,44 @@ class GPUShaderModule(GPUObject):
     Create a shader module using :func:`GPUDevice.create_shader_module`.
     """
 
+    # wgpu.help('shadermodulecompilationinfo', dev=True)
+    # IDL: Promise<GPUCompilationInfo> compilationInfo();
+    def compilation_info(self):
+        """ Get shader compilation info. Always returns empty string at the moment.
+        """
+        return []
 
-class GPUComputePipeline(GPUObject):
+    # wgpu.help('shadermodulecompilationinfo', dev=True)
+    # IDL: Promise<GPUCompilationInfo> compilationInfo();
+    async def compilation_info_async(self):
+        """ Async version of compilation_info()
+        """
+        return self.compilation_info()  # no-cover
+
+
+class PipelineBase(GPUObject):
+    """ Base object for compute and render pipelines.
+    """
+
+    def __init__(self, label, internal, device, layout):
+        super().__init__(label, internal, device)
+        self._layout = layout
+
+    @property
+    def layout(self):
+        """ The the layout of this pipeline.
+        """
+        return self._layout
+
+    # wgpu.help('pipelinebasegetbindgrouplayout', dev=True)
+    # IDL: GPUBindGroupLayout getBindGroupLayout(unsigned long index);
+    def get_bind_group_layout(self, index):
+        """ Get the bind group layout at the given index.
+        """
+        return self._layout._layouts[index]
+
+
+class GPUComputePipeline(PipelineBase):
     """
     A compute pipeline represents a single pipeline for computations (no rendering).
 
@@ -842,7 +949,7 @@ class GPUComputePipeline(GPUObject):
     """
 
 
-class GPURenderPipeline(GPUObject):
+class GPURenderPipeline(PipelineBase):
     """
     A render pipeline represents a single pipeline to draw something
     using a vertex and a fragment shader. The render target can come
@@ -876,7 +983,7 @@ class GPUCommandEncoder(GPUObject):
         """ Record the beginning of a compute pass. Returns a
         :class:`GPUComputePassEncoder` object.
 
-        Parameters:
+        Arguments:
             label (str): A human readable label. Optional.
         """
         raise NotImplementedError()
@@ -896,22 +1003,22 @@ class GPUCommandEncoder(GPUObject):
 
         Arguments:
             label (str): A human readable label. Optional.
-            color_attachements (list of dict): List of color attachement dicts. See below.
-            depth_stencil_attachment (dict): A depth stencil attachement dict. See below. Default None.
+            color_attachments (list of dict): List of color attachment dicts. See below.
+            depth_stencil_attachment (dict): A depth stencil attachment dict. See below. Default None.
             occlusion_query_set: Default None. TODO NOT IMPLEMENTED in wgpu-native.
 
-        Example color attachement:
+        Example color attachment:
 
         .. code-block:: py
 
             {
-                "attachement": texture_view,
+                "attachment": texture_view,
                 "resolve_target": None,  # optional
                 "load_value": (0, 0, 0, 0),  # LoadOp.load or a color
                 "store_op": wgpu.StoreOp.store,  # optional
             }
 
-        Example depth stencil attachement:
+        Example depth stencil attachment:
 
         .. code-block:: py
 
@@ -948,7 +1055,7 @@ class GPUCommandEncoder(GPUObject):
 
         Arguments:
             source (GPUBuffer): A dict with fields: buffer, offset, bytes_per_row, rows_per_image.
-            destination (GPUTexture): A dict with fields: texture, mip_level, array_layer, origin.
+            destination (GPUTexture): A dict with fields: texture, mip_level, origin.
             copy_size (int): The number of bytes to copy.
         """
         raise NotImplementedError()
@@ -959,7 +1066,7 @@ class GPUCommandEncoder(GPUObject):
         """ Copy the contents of a texture (view) to a buffer.
 
         Arguments:
-            source (GPUTexture): A dict with fields: texture, mip_level, array_layer, origin.
+            source (GPUTexture): A dict with fields: texture, mip_level, origin.
             destination (GPUBuffer):  A dict with fields: buffer, offset, bytes_per_row, rows_per_image.
             copy_size (int): The number of bytes to copy.
         """
@@ -971,14 +1078,14 @@ class GPUCommandEncoder(GPUObject):
         """ Copy the contents of a texture (view) to another texture (view).
 
         Arguments:
-            source (GPUTexture): A dict with fields: texture, mip_level, array_layer, origin.
-            destination (GPUTexture):  A dict with fields: texture, mip_level, array_layer, origin.
+            source (GPUTexture): A dict with fields: texture, mip_level, origin.
+            destination (GPUTexture):  A dict with fields: texture, mip_level, origin.
             copy_size (int): The number of bytes to copy.
         """
         raise NotImplementedError()
 
     # wgpu.help('commandencoderpushdebuggroup', dev=True)
-    # IDL: void pushDebugGroup(DOMString groupLabel);
+    # IDL: void pushDebugGroup(USVString groupLabel);
     def push_debug_group(self, group_label):
         """ TODO: not yet available in wgpu-native
         """
@@ -992,7 +1099,7 @@ class GPUCommandEncoder(GPUObject):
         raise NotImplementedError()
 
     # wgpu.help('commandencoderinsertdebugmarker', dev=True)
-    # IDL: void insertDebugMarker(DOMString markerLabel);
+    # IDL: void insertDebugMarker(USVString markerLabel);
     def insert_debug_marker(self, marker_label):
         """ TODO: not yet available in wgpu-native
         """
@@ -1004,7 +1111,7 @@ class GPUCommandEncoder(GPUObject):
         """ Finish recording. Returns a :class:`GPUCommandBuffer` to
         submit to a :class:`GPUQueue`.
 
-        Parameters:
+        Arguments:
             label (str): A human readable label. Optional.
         """
         raise NotImplementedError()
@@ -1038,7 +1145,7 @@ class GPUProgrammablePassEncoder(GPUObject):
         raise NotImplementedError()
 
     # wgpu.help('programmablepassencoderpushdebuggroup', dev=True)
-    # IDL: void pushDebugGroup(DOMString groupLabel);
+    # IDL: void pushDebugGroup(USVString groupLabel);
     def push_debug_group(self, group_label):
         """ Push a named debug group into the command stream.
         """
@@ -1052,7 +1159,7 @@ class GPUProgrammablePassEncoder(GPUObject):
         raise NotImplementedError()
 
     # wgpu.help('programmablepassencoderinsertdebugmarker', dev=True)
-    # IDL: void insertDebugMarker(DOMString markerLabel);
+    # IDL: void insertDebugMarker(USVString markerLabel);
     def insert_debug_marker(self, marker_label):
         """ Insert the given message into the debug message queue.
         """
@@ -1130,7 +1237,8 @@ class GPURenderEncoderBase(GPUProgrammablePassEncoder):
         Arguments:
             buffer (GPUBuffer): The buffer that contains the indices.
             offset (int): The byte offset in the buffer. Default 0.
-            size (int): The number of bytes to use. Default 0.
+            size (int): The number of bytes to use. If zero, the remaining size
+                (after offset) of the buffer is used. Default 0.
         """
         raise NotImplementedError()
 
@@ -1143,7 +1251,8 @@ class GPURenderEncoderBase(GPUProgrammablePassEncoder):
             slot (int): The binding slot for the vertex buffer.
             buffer (GPUBuffer): The buffer that contains the vertex data.
             offset (int): The byte offset in the buffer. Default 0.
-            size (int): The number of bytes to use. Default 0.
+            size (int): The number of bytes to use. If zero, the remaining size
+                (after offset) of the buffer is used. Default 0.
         """
         raise NotImplementedError()
 
@@ -1295,7 +1404,7 @@ class GPURenderBundleEncoder(GPURenderEncoderBase):
     def finish(self, *, label=""):
         """ Finish recording and return a :class:`GPURenderBundle`.
 
-        Parameters:
+        Arguments:
             label (str): A human readable label. Optional.
         """
         raise NotImplementedError()
@@ -1326,6 +1435,41 @@ class GPUQueue(GPUObject):
         """
         raise NotImplementedError()
 
+    # wgpu.help('Buffer', 'Size64', 'queuewritebuffer', dev=True)
+    # IDL: void writeBuffer( GPUBuffer buffer, GPUSize64 bufferOffset, [AllowShared] ArrayBuffer data, optional GPUSize64 dataOffset = 0, optional GPUSize64 size);
+    def write_buffer(self, buffer, buffer_offset, data, data_offset=0, size=None):
+        """ Takes the data contents and schedules a write operation of
+        these contents to the buffer. A snapshot of the data is taken;
+        any changes to the data after this function is called do not
+        affect the buffer contents.
+
+        Arguments:
+            buffer: The :class:`GPUBuffer` object to write to.
+            buffer_offset (int): The offset in the buffer to start writing at.
+            data: The data to write. Must be contiguous.
+            data_offset: The byte offset in the data. Default 0.
+            size: The number of bytes to write. Default all minus offset.
+        """
+        raise NotImplementedError()
+
+    # wgpu.help('Extent3D', 'TextureCopyView', 'TextureDataLayout', 'queuewritetexture', dev=True)
+    # IDL: void writeTexture( GPUTextureCopyView destination, [AllowShared] ArrayBuffer data, GPUTextureDataLayout dataLayout, GPUExtent3D size);
+    def write_texture(self, destination, data, data_layout, size):
+        """ Takes the data contents and schedules a write operation of
+        these contents to the destination texture in the queue. A
+        snapshot of the data is taken; any changes to the data after
+        this function is called do not affect the texture contents.
+
+        Arguments:
+            destination: A dict with fields: "texture" (a texture object),
+                "origin" (a 3-tuple), "mip_level" (an int, default 0).
+            data: The data to write.
+            data_layout: A dict with fields: "offset" (an int, default 0),
+                "bytes_per_row" (an int), "rows_per_image" (an int, default 0).
+            size: A 3-tuple of ints specifying the size to write.
+        """
+        raise NotImplementedError()
+
     # Not implemented, because fences do not yet have a use
     # def createFence(self):
     # def signal(self):
@@ -1336,7 +1480,7 @@ class GPUQueue(GPUObject):
 class GPUSwapChain(GPUObject):
     """
     A swap chain is a placeholder for a texture to be presented to the screen,
-    so that you can provide the corresponding texture view as a color attachement
+    so that you can provide the corresponding texture view as a color attachment
     to :func:`GPUCommandEncoder.begin_render_pass`. The texture view can be
     obtained by using the swap-chain in a with-statement. The swap-chain is
     presented to the screen when the context exits.
